#!/bin/sh
#   Execute a program for X, shows a message if missing
#   and helps maintain a 'last run' menu in fvwm2

#------------------------------------------------------------------------------
# runmsg()
#------------------------------------------------------------------------------
runmsg()
{

    umsg="$1"

    xmessage -geometry '-0+0' -buttons "TFB" "$umsg" &

    if test "$rm_lock" = "Y"
    then
	rm -f "$lockfile"
    fi

    exit 0

} # END: runmsg()

#------------------------------------------------------------------------------
# timemsg() -- show a message that ends after 1 second
#------------------------------------------------------------------------------
timemsg()
{

    umsg="$1"

    if test "$umsg" != ""
    then
	xmessage -geometry '-0+0' -timeout 1 -buttons "OK" "$umsg" &
    fi


} # END: timemsg()

#------------------------------------------------------------------------------
# mk_last() -- Regenerate last run file for FVWM, try and generate the file
#              in reverse order.  As far as I know only jtail can do that
#------------------------------------------------------------------------------
mk_last()
{

    if test -d "$ddir"
    then
	last_file="$ddir/last_run_x.txt"
	last_fmt="$ddir/last_run_x_fmt.txt"
	touch "$last_file" "$last_fmt"
	chmod 644 "$last_file" "$last_fmt"
	pname=`echo $@ | awk '{print $1}'`
	grep -v ^"$pname" <  "$last_file" \
	    | sed 's/[\t ]*$//g'          \
	    | sed 's/^[\t ]*//'           \
	    | grep -v '^$'                \
	    | tail -14                    \
	    > "$last_file".OLD
	echo "$@" >> "$last_file".OLD
	mv "$last_file".OLD "$last_file"
	type jtail > /dev/null 2>&1
	if test "$?" -eq "0"
	then
	    #--- this reverse list entries, only nice to have
	    jtail -n 15 -r "$last_file" > "$last_fmt"
	else
	    cp "$last_file" "$last_fmt"
	fi
	chmod 644 "$last_file" "$last_fmt"
    fi

} # END: mk_last()

###############################################################################
# main
###############################################################################
prog="$1"

fhome=$HOME/.fvwm
ddir=$fhome/data
lockfile="$ddir/lock.xrun_prog.lock"
rm_lock="N"

#------------------------------------------------------------------------------
# validate command
#------------------------------------------------------------------------------
type "$prog" > /dev/null 2>&1
err_code="$?"
if test "$err_code" -ne "0"
then
    runmsg "Program $prog cannot be located"
fi

#------------------------------------------------------------------------------
# log the command and validate lock
# lock file needed to avoid collisions
#------------------------------------------------------------------------------
hour=`date "+%Y%m%d_%H"`

if test -f "$lockfile"
then
    now=`head -n 1 < "$lockfile" | cut -c '1-11'`
    #--- if the lock file is old from a different hour, 
    #    process as if it does not exist
    if test "$hour" = "$now"
    then
	rm_lock="N"
    else
	rm_lock="Y"
	date "+%Y%m%d_%H%M%S" > "$lockfile"
	echo "$@" >> "$lockfile"
    fi
else
    rm_lock="Y"
    date "+%Y%m%d_%H%M%S" > "$lockfile"
    echo "$@" >> "$lockfile"
fi

if test "$rm_lock" = "Y"
then
    mk_last $@ &
    mk_last_fb $@ &
fi

#------------------------------------------------------------------------------
# process the command
#------------------------------------------------------------------------------
timemsg "STARTING: $@"

if test "$rm_lock" = "Y"
then
    rm -f "$lockfile"
fi

exec $@ &

exit 0
